#Class chanels
#Contains all known channels
class Channels:

    channels = {'SMS',
                'Whatsapp',
                'Push',
                'RCS',
                'Viber',
                'Voice',
                'iMessage',
                'Line',
                'Twitter',
                }

    def Check_Channels(self, allowedChannels):
        for allowedChannel in allowedChannels:
            if(not allowedChannel in self.channels):
                print(allowedChannel + ' is not a valid Channel name. Check Spelling?')
                return False
        return True


#Contains all Gateways

class Gateways:
    Global = 'https://gw.cmtelecom.com/v1.0/message'    #Global Endpoint URL
    CN = 'https://gw-cn.cmtelecom.com/v1.0/message'     #China Mainland Endpoint URL
    UK = 'https://gw-uk.cmtelecom.com/v1.0/message'     #London Endpoint URL
    ZA = 'https://gw.cmtelecom.co.za/v1.0/message'      #South Africa Endpoint URL
    HK = 'https://gw-hk.cmtelecom.com/v1.0/message'     #Hongkong Endpoint URL



class TextClientStatusCodes:
    OK = 0
    AUTHENTICATION_FAILED = 101
    BALANCE_INSUFFICIENT = 102
    APIKEY_INCORRECT = 103
    REQUEST_NOT_ALL_SENT = 201
    REQUEST_MALFORMED = 202
    MSG_ARRAY_INCORRECT = 203
    MSG_INVALID_FROM = 301
    MSG_INVALID_TO = 302
    MSG_INVALID_MSISDN = 303
    MSG_INVALID_BODY = 304
    MSG_INVALID_FIELD = 305
    SPAM = 401
    BLACKLISTED = 402
    REJECTED = 403
    INTERNAL_ERROR = 500
    UNKNOWN = 999


from CMText.Gateways import Gateways
from CMText.Message import Message
import json
import requests


class TextClient:
    gateway = ''
    apikey = ''
    messages = []
    MESSAGES_MAXIMUM = 100
    VERSION = '1.0'

    # Initialize Client with defaul Gateway Gateways.Global
    def __init__(self, apikey, gateway=Gateways.Global):
        self.apikey = apikey
        self.gateway = gateway

    # Send 1 message to one or multiple locations
    def SendSingleMessage(self, message, from_, to=[], reference=None, allowedChannels=None):
        self.messages = []
        self.messages.append(Message(message, from_=from_, to=to, reference=reference, allowedChannels=allowedChannels))
        self.send()

    # Add a message to the list
    def AddMessage(self, message, from_='', to=[], reference=None, allowedChannels=None):
        self.messages.append(Message(message, from_=from_, to=to, reference=reference, allowedChannels=allowedChannels))

    # Add a rich message to the list
    def AddRichMessage(self, message, media, from_='', to=[], reference=None, allowedChannels=None):
        self.messages.append(Message(message, media=media, from_=from_, to=to, reference=reference, allowedChannels=allowedChannels))

    # Send all messages in the list
    def send(self):
        if len(self.messages) == 0:
            print('No messages in the queue')
            return
        if len(self.messages) > self.MESSAGES_MAXIMUM:
            print('Messages exceeds MESSAGES_MAXIMUM')
            return

        # Set data for post
        data = self.encodeData(self.messages)

        # Set headers for post
        headers = {
             "Content-Type": "application/json; charset=utf-8",
             "Content-Length": str(len(data)),
             'X-CM-SDK': 'text-sdk-python-' + self.VERSION
         }

        # Send the message
        try:
            response = requests.post("https://gw.cmtelecom.com/v1.0/message", data=data, headers=headers)
            print(data)
        except Exception as e:
            print(e)

        # Clear messages
        self.messages = []
        # Return response
        return response

    # Method to encode Data, Gateway accepts this format
    def encodeData(self, messages):
        # Set productToken
        data = {"messages": {"authentication":{"producttoken": self.apikey}}}
        data['messages']['msg'] = []


        # For each message do this
        for message in messages:
            # List all recipients
            to = []
            for toItem in message.to:
                to = to + [{'number': toItem}]

            # Create message container
            temp = {"allowedChannels": message.allowedChannels,
                    "from": message.from_,
                    "to": to,
                    "body": {
                        "type": message.type,
                        "content": message.body
                    }
                    }

            # If message is rich
            if message.richContent is not None:
                temp["richContent"] = {
                    "conversation": [{
                        "text": message.body
                    },
                    {
                        "media": message.richContent,
                    }]
                }

            data['messages']['msg'] = data['messages']['msg'] + [temp]


        # Json encode the data
        data = json.dumps(data)
        return data



from CMText.MessageBodyTypes import MessageBodyTypes
from CMText.Channels import Channels

class Message:
    body = ''
    type = ''
    customgrouping3 = ''
    from_ = ''
    reference = ''
    to = []
    minimumNumberOfMessageParts = 1
    maximumNumberOfMessageParts = 8
    hybridAppKey = ''
    allowedChannels = ['SMS']
    richContent = None
    SENDER_FALLBACK = 'cm.com'
    MESSAGEPARTS_MINIMUM = 1
    MESSAGEPARTS_MAXIMUM = 8
    RECIPIENTS_MAXIMUM  = 1000

    # init function of class Message
    def __init__(self, body='', type=MessageBodyTypes.AUTO, from_=None, to=[], reference=None, allowedChannels=None, media=None):
        self.body = body
        self.type = type
        if from_ is not None:
            self.from_ = from_
        else:
            self.from_ = self.SENDER_FALLBACK

        # if allowedChannels is set and it exists in Channels:
        ch = Channels()
        if((allowedChannels != None) and ch.Check_Channels(allowedChannels=allowedChannels)):
            self.allowedChannels = allowedChannels

        # if media is not None
        if media is not None:
            self.richContent = media

        self.reference = reference
        self.AddRecipients(recipients=to)

        self.minimumNumberOfMessageParts = self.MESSAGEPARTS_MINIMUM
        self.maximumNumberOfMessageParts = self.MESSAGEPARTS_MAXIMUM

        self.customgrouping3 = 'text-sdk-python-' + '1.0' # + TextClient.VERSION #find version Texclient

    # add an array of recipients
    def AddRecipients(self, recipients=[]):
        # check if total recipients exceeds RECIPIENTS_MAXIMUM
        if(len(self.to) + len(recipients) > self.RECIPIENTS_MAXIMUM):
            print('Maximum amount of Recipients exceeded. (' + str(self.RECIPIENTS_MAXIMUM) + ')')
        else:
            self.to = self.to + recipients